@@ -1,10 +1,12 @@
 import os.path
 import warnings
 
-import satstac
+import pystac
 from intake.catalog import Catalog
 from intake.catalog.local import LocalCatalogEntry
+from intake.source import DataSource
 from pkg_resources import get_distribution
+from pystac.extensions.eo import EOExtension
 
 __version__ = get_distribution('intake_stac').version
 
@@ -26,6 +28,7 @@ drivers = {
     'image/x.geotiff': 'rasterio',
     'image/tiff; application=geotiff': 'rasterio',
     'image/tiff; application=geotiff; profile=cloud-optimized': 'rasterio',  # noqa: E501
+    'image/tiff': 'rasterio',
     'image/jp2': 'rasterio',
     'image/png': 'xarray_image',
     'image/jpg': 'xarray_image',
@@ -36,6 +39,8 @@ drivers = {
     'application/json': 'textfiles',
     'application/geo+json': 'geopandas',
     'application/geopackage+sqlite3': 'geopandas',
+    'application/vnd+zarr': 'zarr',
+    'application/xml': 'textfiles',
 }
 
 
@@ -50,23 +55,25 @@ class AbstractStacCatalog(Catalog):
 
         Parameters
         ----------
-        stac_obj: stastac.Thing
-            A satstac.Thing pointing to a STAC object
+        stac_obj: stastac.STACObject
+            A pystac.STACObject pointing to a STAC object
         kwargs : dict, optional
             Passed to intake.Catalog.__init__
         """
         if isinstance(stac_obj, self._stac_cls):
             self._stac_obj = stac_obj
         elif isinstance(stac_obj, str):
-            self._stac_obj = self._stac_cls.open(stac_obj)
+            self._stac_obj = self._stac_cls.from_file(stac_obj)
         else:
             raise ValueError('Expected %s instance, got: %s' % (self._stac_cls, type(stac_obj)))
 
         metadata = self._get_metadata(**kwargs.pop('metadata', {}))
-
         try:
             name = kwargs.pop('name', self._stac_obj.id)
         except AttributeError:
+            # Not currently tested.
+            # ItemCollection does not require an id
+            # Unclear what the state of ItemCollection is.
             name = str(type(self._stac_obj))
 
         super().__init__(name=name, metadata=metadata, **kwargs)
@@ -83,7 +90,7 @@ class AbstractStacCatalog(Catalog):
         kwargs : dict, optional
             Passed to intake.Catolog.__init__
         """
-        stac_obj = cls._stac_cls.open(url)
+        stac_obj = cls._stac_cls.from_file(url)
         return cls(stac_obj, **kwargs)
 
     def _get_metadata(self, **kwargs):
@@ -102,71 +109,33 @@ class AbstractStacCatalog(Catalog):
 
 class StacCatalog(AbstractStacCatalog):
     """
-    Intake Catalog represeting a STAC Catalog
-    https://github.com/radiantearth/stac-spec/blob/master/catalog-spec/catalog-spec.md
-
-    A Catalog that references a STAC catalog at some URL
-    and constructs an intake catalog from it, with opinionated
-    choices about the drivers that will be used to load the datasets.
-    In general, the drivers are:
-
-        - netcdf
-        - rasterio
-        - xarray_image
-        - textfiles
+    Maps Intake Catalog to a STAC Catalog
+    https://pystac.readthedocs.io/en/latest/api.html?#catalog-spec
     """
 
     name = 'stac_catalog'
-    _stac_cls = satstac.Catalog
+    _stac_cls = pystac.Catalog
 
     def _load(self):
         """
         Load the STAC Catalog.
         """
-        subcatalog = None
-        # load first sublevel catalog(s)
-        for subcatalog in self._stac_obj.children():
+        for subcatalog in self._stac_obj.get_children():
+            if isinstance(subcatalog, pystac.Collection):
+                # Collection subclasses Catalog, so check it first
+                driver = StacCollection
+            else:
+                driver = StacCatalog
+
             self._entries[subcatalog.id] = LocalCatalogEntry(
                 name=subcatalog.id,
                 description=subcatalog.description,
-                driver=StacCatalog,
+                driver=driver,  # recursive
                 catalog=self,
-                args={'stac_obj': subcatalog.filename},
+                args={'stac_obj': subcatalog.get_self_href()},
             )
 
-        if subcatalog is None:
-            # load items under last catalog
-            for item in self._stac_obj.items():
-                self._entries[item.id] = LocalCatalogEntry(
-                    name=item.id,
-                    description='',
-                    driver=StacItem,
-                    catalog=self,
-                    args={'stac_obj': item},
-                )
-
-    def _get_metadata(self, **kwargs):
-        """
-        Keep copy of all STAC JSON except for links
-        """
-        metadata = self._stac_obj._data.copy()
-        del metadata['links']
-        return metadata
-
-
-class StacItemCollection(AbstractStacCatalog):
-    """
-    Intake Catalog represeting a STAC ItemCollection
-    """
-
-    name = 'stac_item_collection'
-    _stac_cls = satstac.ItemCollection
-
-    def _load(self):
-        """
-        Load the STAC Item Collection.
-        """
-        for item in self._stac_obj:
+        for item in self._stac_obj.get_items():
             self._entries[item.id] = LocalCatalogEntry(
                 name=item.id,
                 description='',
@@ -176,7 +145,109 @@ class StacItemCollection(AbstractStacCatalog):
             )
 
     def _get_metadata(self, **kwargs):
-        return kwargs
+        """
+        Keep copy of all STAC JSON except for links
+        """
+        # NOTE: why not links?
+        metadata = self._stac_obj.to_dict()
+        del metadata['links']
+        return metadata
+
+
+class StacCollection(StacCatalog):
+    """
+    Maps Intake Catalog to a STAC Collection
+    https://pystac.readthedocs.io/en/latest/api.html#collection-spec
+
+    Collections have a number of properties that Catalogs do not, most notably
+    the spatial and temporal extents. This is currently a placeholder for
+    future Collection-specific attributes and methods.
+    """
+
+    name = 'stac_catalog'
+    _stac_cls = pystac.Collection
+
+    def get_asset(
+        self,
+        key,
+        storage_options=None,
+        merge_asset_storage_options=True,
+        merge_asset_open_kwargs=True,
+        **kwargs,
+    ):
+        r"""
+        Get a datasource for a collection-level asset.
+
+        Parameters
+        ----------
+        key : str, optional
+            The asset key to use if multiple Zarr assets are provided.
+        storage_options : dict, optional
+            Additional arguments for the backend fsspec filesystem.
+        merge_asset_storage_option : bool, default True
+            Whether to merge the storage options provided by the asset under the
+            ``xarray:storage_options`` key with `storage_options`.
+        merge_asset_open_kwargs : bool, default True
+            Whether to merge the keywords provided by the asset under the
+            ``xarray:open_kwargs`` key with ``**kwargs``.
+        **kwargs
+            Additional keyword options are provided to the loader, for example ``consolidated=True``
+            to pass to :meth:`xarray.open_zarr`.
+
+        Notes
+        -----
+        The Media Type of the asset will be used to determine how to load the data.
+
+        Returns
+        -------
+        DataSource
+            The dataset described by the asset loaded into a dask-backed object.
+        """
+        try:
+            asset = self._stac_obj.assets[key]
+        except KeyError:
+            raise KeyError(
+                f'No asset named {key}. Should be one of {list(self._stac_obj.assets)}'
+            ) from None
+
+        storage_options = storage_options or {}
+        if merge_asset_storage_options:
+            asset_storage_options = asset.extra_fields.get('xarray:storage_options', {})
+            storage_options.update(asset_storage_options)
+
+        if merge_asset_open_kwargs:
+            asset_open_kwargs = asset.extra_fields.get('xarray:open_kwargs', {})
+            kwargs.update(asset_open_kwargs)
+
+        return StacAsset(key, asset)(storage_options=storage_options, **kwargs)
+
+
+class StacItemCollection(AbstractStacCatalog):
+    """
+    Maps ItemCollection returned from a STAC API to Intake (Sub)Catalog
+    https://github.com/radiantearth/stac-api-spec/tree/master/fragments/itemcollection
+
+    Note search results often use the single file stac extension:
+    https://pystac.readthedocs.io/en/latest/api.html?#single-file-stac-extension
+    """
+
+    name = 'stac_itemcollection'
+    _stac_cls = pystac.ItemCollection
+
+    def _load(self):
+        """
+        Load the STAC Item Collection.
+        """
+        # if not self._stac_obj.ext.implements('single-file-stac'):
+        #     raise ValueError("StacItemCollection requires 'single-file-stac' extension")
+        for item in self._stac_obj.items:
+            self._entries[item.id] = LocalCatalogEntry(
+                name=item.id,
+                description='',
+                driver=StacItem,
+                catalog=self,
+                args={'stac_obj': item},
+            )
 
     def to_geopandas(self, crs=None):
         """
@@ -201,63 +272,40 @@ class StacItemCollection(AbstractStacCatalog):
             )
 
         if crs is None:
-            crs = {'init': 'epsg:4326'}
-        gf = gpd.GeoDataFrame.from_features(self._stac_obj.geojson(), crs=crs)
+            crs = 'epsg:4326'
+        gf = gpd.GeoDataFrame.from_features(self._stac_obj.to_dict(), crs=crs)
         return gf
 
 
-class StacCollection(AbstractStacCatalog):
-    """
-    Intake Catalog represeting a STAC Collection
-    https://github.com/radiantearth/stac-spec/blob/master/collection-spec/collection-spec.md
-    """
-
-    name = 'stac_collection'
-    _stac_cls = satstac.Collection
-
-    def _load(self):
-        """
-        Load the STAC Collection.
-        """
-        for item in self._stac_obj.items():
-            self._entries[item.id] = LocalCatalogEntry(
-                name=item.id,
-                description='',
-                driver=StacItem,
-                catalog=self,
-                args={'stac_obj': item.filename},
-            )
-
-    def _get_metadata(self, **kwargs):
-        metadata = {}
-        for attr in [
-            'title',
-            'version',
-            'keywords',
-            'license',
-            'providers',
-            'extent',
-        ]:
-            metadata[attr] = getattr(self._stac_obj, attr, None)
-        metadata.update(kwargs)
-        return metadata
-
-
 class StacItem(AbstractStacCatalog):
     """
-    Intake Catalog represeting a STAC Item
-    https://github.com/radiantearth/stac-spec/blob/master/item-spec/item-spec.md
+    Maps STAC Item to Intake (Sub)Catalog
+    https://pystac.readthedocs.io/en/latest/api.html#item-spec
     """
 
     name = 'stac_item'
-    _stac_cls = satstac.Item
+    _stac_cls = pystac.Item
+
+    def __getitem__(self, key):
+        result = super().__getitem__(key)
+        # TODO: handle non-string assets?
+        asset = self._entries[key]
+        storage_options = asset._stac_obj.extra_fields.get('xarray:storage_options', {})
+        open_kwargs = asset._stac_obj.extra_fields.get('xarray:open_kwargs', {})
+
+        if isinstance(result, DataSource):
+            kwargs = result._captured_init_kwargs
+            kwargs = {**kwargs, **dict(storage_options=storage_options), **open_kwargs}
+            result = result(*result._captured_init_args, **kwargs)
+
+        return result
 
     def _load(self):
         """
         Load the STAC Item.
         """
         for key, value in self._stac_obj.assets.items():
-            self._entries[key] = StacEntry(key, value)
+            self._entries[key] = StacAsset(key, value)
 
     def _get_metadata(self, **kwargs):
         metadata = self._stac_obj.properties.copy()
@@ -268,31 +316,14 @@ class StacItem(AbstractStacCatalog):
 
     def _get_band_info(self):
         """
-        helper function for stack_bands
+        Return list of band info dictionaries (name, common_name, etc.)...
         """
-        # Try to get band-info at Collection then Item level
         band_info = []
-        try:
-            collection = self._stac_obj.collection()
-            if 'item-assets' in collection._data.get('stac_extensions'):
-                for val in collection._data['item_assets'].values():
-                    if 'eo:bands' in val:
-                        band_info.append(val.get('eo:bands')[0])
-            else:
-                band_info = collection.summaries['eo:bands']
-
-        except KeyError:
-            for val in self._stac_obj.assets.values():
-                if 'eo:bands' in val:
-                    band_info.append(val.get('eo:bands')[0])
-        finally:
-            if not band_info:
-                raise AttributeError(
-                    'Unable to parse "eo:bands" information from STAC Collection or Item Assets'
-                )
+        for band in EOExtension.ext(self._stac_obj).bands:
+            band_info.append(band.to_dict())
         return band_info
 
-    def stack_bands(self, bands):
+    def stack_bands(self, bands, path_as_pattern=None, concat_dim='band'):
         """
         Stack the listed bands over the ``band`` dimension.
 
@@ -313,26 +344,33 @@ class StacItem(AbstractStacCatalog):
 
         Returns
         -------
-        StacEntry with mapping of Asset names to Xarray bands
+        StacAsset with mapping of Asset names to Xarray bands
 
-        Example
-        -------
+        Examples
+        --------
         stack = item.stack_bands(['nir','red'])
         da = stack(chunks=dict(band=1, x=2048, y=2048)).to_dask()
-        """
 
-        if 'eo' not in self._stac_obj._data['stac_extensions']:
-            raise AttributeError('STAC Item must implement "eo" extension to use this method')
+        stack = item.stack_bands(['B4','B5'], path_as_pattern='{band}.TIF')
+        da = stack(chunks=dict(band=1, x=2048, y=2048)).to_dask()
+        """
+        if not EOExtension.has_extension(self._stac_obj):
+            raise ValueError('STAC Item must implement "eo" extension to use this method')
 
         band_info = self._get_band_info()
-        item = {'concat_dim': 'band', 'urlpath': []}
+        configDict = {}
+        metadatas = {}
         titles = []
+        hrefs = []
         types = []
         assets = self._stac_obj.assets
         for band in bands:
             # band can be band id, name or common_name
             if band in assets:
-                info = next((b for b in band_info if b.get('id', b.get('name')) == band), None,)
+                info = next(
+                    (b for b in band_info if b.get('id', b.get('name')) == band),
+                    None,
+                )
             else:
                 info = next((b for b in band_info if b.get('common_name') == band), None)
                 if info is not None:
@@ -347,67 +385,81 @@ class StacItem(AbstractStacCatalog):
                     f'{band} not found in list of eo:bands in collection.'
                     f'Valid values: {sorted(list(set(valid_band_names)))}'
                 )
-
-            value = assets.get(band)
-            band_type = value.get('type')
-            types.append(band_type)
-
-            href = value.get('href')
-            pattern = href.replace(band, '{band}')
-            if 'path_as_pattern' not in item:
-                item['path_as_pattern'] = pattern
-            elif item['path_as_pattern'] != pattern:
-                raise ValueError(
-                    f'Stacking failed: {href} does not contain '
-                    'band info in a fixed section of the url'
-                )
-
+            asset = assets.get(band)
+            metadatas[band] = asset.to_dict()
             titles.append(band)
-            item['urlpath'].append(href)
+            types.append(asset.media_type)
+            hrefs.append(asset.href)
 
         unique_types = set(types)
         if len(unique_types) != 1:
             raise ValueError(
                 f'Stacking failed: bands must have type, multiple found: {unique_types}'
             )
-        else:
-            item['type'] = types[0]
 
-        item['title'] = ', '.join(titles)
-        return StacEntry('_'.join(bands), item, stacked=True)
+        configDict['name'] = '_'.join(bands)
+        configDict['description'] = ', '.join(titles)
+        configDict['args'] = dict(
+            chunks={}, concat_dim=concat_dim, path_as_pattern=path_as_pattern, urlpath=hrefs
+        )
+        configDict['metadata'] = metadatas
+
+        return CombinedAssets(configDict)
+
+    def _yaml(self):
+        data = {'metadata': {}, 'sources': {}}
+        data['metadata'].update(self.metadata)
+        for key, source in self.items():
+            data['sources'][key] = source._yaml()['sources']['stac_asset']
+            data['sources'][key]['direct_access'] = 'allow'
+            data['sources'][key]['metadata'].pop('catalog_dir', None)
+        return data
 
 
-class StacEntry(LocalCatalogEntry):
+class StacAsset(LocalCatalogEntry):
     """
-    A class representing a STAC catalog Entry
+    Maps 1 STAC Item Asset to 1 Intake Catalog Entry
+    https://pystac.readthedocs.io/en/latest/api.html#asset
     """
 
-    def __init__(self, key, item, stacked=False):
+    name = 'stac_asset'
+    _stac_cls = pystac.item.Asset
+
+    def __init__(self, key, asset):
         """
         Construct an Intake catalog 'Source' from a STAC Item Asset.
+        asset = pystac.item.Asset
         """
-        driver = self._get_driver(item)
-
-        default_plot = self._get_plot(item)
-        if default_plot:
-            item['plots'] = default_plot
+        self._stac_obj = asset
+        driver = self._get_driver(asset)
 
         super().__init__(
             name=key,
-            description=item.get('title', key),
+            description=asset.title,
             driver=driver,
-            direct_access=True,
-            args=self._get_args(item, driver, stacked=stacked),
-            metadata=item,
+            direct_access='allow',
+            args=self._get_args(asset, driver),
+            metadata=self._get_metadata(asset),
         )
 
-    def _get_plot(self, item):
+    def _get_metadata(self, asset):
+        """
+        Copy STAC Asset Metadata and setup default plot
+        """
+        metadata = asset.to_dict()
+        default_plot = self._get_plot(asset)
+        if default_plot:
+            metadata['plots'] = default_plot
+
+        return metadata
+
+    def _get_plot(self, asset):
         """
         Default hvplot plot based on Asset mimetype
         """
-        # NOTE: consider geojson, parquet, hdf defaults in future
+        # NOTE: consider geojson, parquet, hdf defaults in future...
         default_plot = None
-        type = item.get('type', None)  # also some assets do not have 'type'
+        type = asset.media_type
         if type:
             if type in ['image/jpeg', 'image/jpg', 'image/png']:
                 default_plot = dict(
@@ -439,34 +491,60 @@ class StacEntry(LocalCatalogEntry):
 
         return default_plot
 
-    def _get_driver(self, entry):
-
-        entry_type = entry.get('type')
+    def _get_driver(self, asset):
+        """
+        Assign intake driver for data I/O
+        """
+        entry_type = asset.media_type
 
         if entry_type in ['', 'null', None]:
 
-            suffix = os.path.splitext(entry['href'])[-1]
+            suffix = '.tif'
+            if asset.media_type:
+                suffix = os.path.splitext(asset.media_type)[-1]
             if suffix in ['.nc', '.h5', '.hdf']:
-                entry['type'] = 'application/netcdf'
+                asset.media_type = 'application/netcdf'
                 warnings.warn(
-                    f'STAC Asset "type" missing, assigning {entry_type} based on href suffix {suffix}:\n{entry}'  # noqa: E501
+                    f'STAC Asset "type" missing, assigning {entry_type} based on href suffix {suffix}:\n{asset.media_type}'  # noqa: E501
                 )
             else:
-                entry['type'] = default_type
+                asset.media_type = default_type
                 warnings.warn(
-                    f'STAC Asset "type" missing, assuming default type={default_type}:\n{entry}'
+                    f'STAC Asset "type" missing, assuming default type={default_type}:\n{asset}'  # noqa: E501
                 )
-            entry_type = entry.get('type')
-            print(entry_type)
+            entry_type = asset.media_type
 
         # if mimetype not registered try rasterio driver
         driver = drivers.get(entry_type, default_driver)
 
         return driver
 
-    def _get_args(self, entry, driver, stacked=False):
-        args = entry if stacked else {'urlpath': entry.get('href')}
+    def _get_args(self, asset, driver):
+        """
+        Optional keyword arguments to pass to intake driver
+        """
+        args = {'urlpath': asset.href}
         if driver in ['netcdf', 'rasterio', 'xarray_image']:
+            # NOTE: force using dask?
             args.update(chunks={})
 
         return args
+
+
+class CombinedAssets(LocalCatalogEntry):
+    """
+    Maps multiple STAC Item Assets to 1 Intake Catalog Entry
+    """
+
+    def __init__(self, configDict):
+        """
+        configDict = intake Entry dictionary from stack_bands() method
+        """
+        super().__init__(
+            name=configDict['name'],
+            description=configDict['description'],
+            driver='rasterio',  # stack_bands only relevant to rasterio driver?
+            direct_access=True,
+            args=configDict['args'],
+            metadata=configDict['metadata'],
+        )
